<?php

namespace Shavshukov\EventAppBase;

use Shavshukov\EventAppBase\FileReaders\FileReaderInterface;

use React\EventLoop\LoopInterface;

interface ApplicationInterface
{
    /**
     * Get application description
     * 
     * @return string
     */
    public function description(): string;
    
    /**
     * Get path to the application
     * 
     * @return string
     */
    public function getAppPath(): string;
    
    /**
     * Get an instance of loop, used for application events
     * 
     * @return \React\EventLoop\LoopInterface
     */
    public function getLoop(): LoopInterface;
    
    /**
     * Get application file reader
     * 
     * @return \Shavshukov\EventAppBase\FileReaders\FileReaderInterface
     */
    public function getFileReader(): FileReaderInterface;
    
    /**
     * Get Config instance or a property from config
     * 
     * @param string|null $configPath
     * @param mixed $defaultValue
     * @return \Shavshukov\EventAppBase\Interfaces\ModelInterface|mixed
     */
    public function config(string $configPath = null, $defaultValue = null);
    
    /**
     * Resolve a service by name, or if the name is not provided, return a ServiceContainer instance
     * 
     * @param string $serviceName
     * @param array $arguments
     * @return \Shavshukov\EventAppBase\Service\Container|mixed
     */
    public function service(string $serviceName = null, ...$arguments);
    
    /**
     * Run loop
     * 
     * @return null
     */
    public function run();
    
    /**
     * Stop loop
     * 
     * @return null
     */
    public function stop();
    
}
