<?php

namespace Shavshukov\EventAppBase\Model;

use Shavshukov\EventAppBase\MicroEvent;
use Closure;

trait ModelEventsTrait
{
    /**
     * Model booting registration
     *
     * @var array
     */
    protected static $booted = [];
    
    /**
     * Register 'booting' event handler
     * 
     * @param \Closure $handler
     */
    public static function booting(Closure $handler)
    {
        MicroEvent::registerEvent(static::class, 'booting', $handler);
    }
    
    /**
     * Register 'booted' event handler
     * 
     * @param \Closure $handler
     */
    public static function booted(Closure $handler)
    {
        MicroEvent::registerEvent(static::class, 'booted', $handler);
    }
    
    /**
     * Register 'loading' event handler
     * 
     * @param \Closure $handler
     */
    public static function loading(Closure $handler)
    {
        MicroEvent::registerEvent(static::class, 'loading', $handler);
    }
    
    /**
     * Register 'loaded' event handler
     * 
     * @param \Closure $handler
     */
    public static function loaded(Closure $handler)
    {
        MicroEvent::registerEvent(static::class, 'loaded', $handler);
    }
    
    /**
     * Register 'saving' event handler
     * 
     * @param \Closure $handler
     */
    public static function saving(Closure $handler)
    {
        MicroEvent::registerEvent(static::class, 'saving', $handler);
    }
    
    /**
     * Register 'saved' event handler
     * 
     * @param \Closure $handler
     */
    public static function saved(Closure $handler)
    {
        MicroEvent::registerEvent(static::class, 'saved', $handler);
    }
    
    /**
     * Load data from the storage
     * 
     * @return bool
     */
    public function load(): bool
    {
        if (MicroEvent::fireEvent(static::class, 'loading', $this) !== false) {
            $result = parent::load();
            
            if (MicroEvent::fireEvent(static::class, 'loaded', $this, $result) === false) {
                return false;
            }
            
            return $result;
        }
        
        return false;
    }
    
    /**
     * Load the data from storage in a non-blocking way
     * 
     * @return \React\Promise\PromiseInterface<bool>
     */
    public function loadAsync(): \React\Promise\PromiseInterface
    {
        if (MicroEvent::fireEvent(static::class, 'loading', $this) !== false) {
            return parent::loadAsync()->then(function ($result) {
                if (MicroEvent::fireEvent(static::class, 'loaded', $this, $result) === false) {
                    return false;
                }
                
                return $result;
            });
        }
        
        return \React\Promise\resolve(false);
    }
    
    /**
     * Save data into the storage, if modified
     * 
     * @return bool
     */
    public function save(): bool
    {
        if (MicroEvent::fireEvent(static::class, 'saving', $this) !== false) {
            $result = parent::save();
            
            if (MicroEvent::fireEvent(static::class, 'saved', $this, $result) === false) {
                return false;
            }
            
            return $result;
        }
        
        return false;
    }
    
    /**
     * Boot events trait if it wasn't already
     *
     * @return null
     */
    protected function bootEventsIfNotBooted()
    {
        if (!isset(static::$booted[static::class])) {
            static::$booted[static::class] = true;

            MicroEvent::fireEvent(static::class, 'booting', false);

            static::bootEvents();

            MicroEvent::fireEvent(static::class, 'booted', false);
        }
    }
    
}
